#include <windows.h>
#include <stdio.h>


// Define how many items the stack can hold.
#define SIZE 11


// Here is the data structure that defines a stack type buffer.
// The array has SIZE+2 entries so that it can hold SIZE pieces
// of data, a special character to denote the end of the stack
// space, and a null charater ('\0') to denote the end of a string
// (the buffer will be printed out as one whole string).
typedef struct
{
   char buffer[SIZE+2];
   unsigned int top;   // top of stack index
} Buffer;


// Prototype for the printBuffer function (defined below).
void printBuffer(Buffer* buffer_p, int bufferNo);


// Function for pushing an item onto the stack data structure.
void putInBuffer(char item, Buffer* buffer_p, int bufferNo)
{
   // Push an item on the stack and adjust the top index.
   (buffer_p->buffer)[buffer_p->top] = item;
   (buffer_p->top)++;

   // Print the stack.
   printBuffer(buffer_p, bufferNo);

}//putInBuffer


// Function for popping an item off of the stack data structure.
char takeFromBuffer(Buffer* buffer_p, int bufferNo)
{
   char result;

   // Remove the item from the top of the stack (and store that
   // item in result), then adjust the top of stack index. Also,
   // put the character '0' into the buffer where the top of stack
   // item used to be, so that the stack looks better when it is
   // printed out.
   (buffer_p->top)--;
   result = (buffer_p->buffer)[buffer_p->top];
   (buffer_p->buffer)[buffer_p->top] = '0'; // mark space as empty

   // print the stack
   printBuffer(buffer_p, bufferNo);

   return result;
}//takeFromBuffer


// function that prints a stack buffer in its own column in the output
void printBuffer(Buffer* buffer_p, int bufferNo)
{
   // a string of 70 spaces
   char* spaces = "                                                                      ";

   // print the stack and show the top of stack index
   printf("%.*s%s\n%.*s%.*s^\n", bufferNo*(SIZE+2), spaces, buffer_p->buffer,
                                 bufferNo*(SIZE+2), spaces, buffer_p->top, spaces);
}//printBuffer


// function that initialzes all of the buffers
void initializeBuffers(Buffer* buffers, int bufferCount)
{
   int i, j;

   // initialize the stack buffers to be empty
   for (i = 0; i < bufferCount; i++)
      for (j = 0; j < SIZE; j++)
        (buffers[i].buffer)[j] = '0';
   // mark the end of the stack with a special character
   for (i = 0; i < bufferCount; i++)
      (buffers[i].buffer)[SIZE] = '*';
   // null terminate each  of the buffers
   for (i = 0; i < bufferCount; i++)
      (buffers[i].buffer)[SIZE+1] = '\0';

   // initialize the stack buffer indexes
   for (i = 0; i < bufferCount; i++)
      buffers[i].top = 0;

   printf("Here is the initial state of the buffers.\n\n");
   // print column headings (each column is SIZE+2 characters wide)
   for (i = 0; i < bufferCount; i++)
      printf("buffers[%d]%.*s", i, (SIZE+2)-10, "          ");
   printf("\n\n");
   for (i = 0; i < bufferCount; i++)  // print each buffer
      printBuffer(&(buffers[i]), i);

}//initializeBuffers


/*
    Use this function to print out meaningful error messages
*/
void printError(char* functionName)
{   LPVOID lpMsgBuf;
    int error_no;
    error_no = GetLastError();
    FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
    );
    // Display the string.
    fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
    fprintf(stderr, lpMsgBuf);
    // Free the buffer.
    LocalFree( lpMsgBuf );
    //ExitProcess(1);  // terminate the program
}//printError